@@ -1,6 +1,6 @@
 from blazeutils.datastructures import BlankObject
 import formencode
-from savalidation._internal import ValidationHandler, ClassMutator
+from savalidation.validators import EntityLinker, ValidatorBase
 from sqlalchemy import sql as sasql
 
 from compstack.sqlalchemy import db
@@ -18,20 +18,19 @@ class _UniqueValidator(formencode.validators.FancyValidator):
 
     def validate_python(self, value, state):
         existing_record = self.cls.get_by(**{self.fieldname:value})
-        if existing_record and existing_record is not state.instance:
+        if existing_record and existing_record is not state.entity:
             raise formencode.Invalid(self.message('notunique', state), value, state)
-        return
 
-class _UniqueValidationHandler(ValidationHandler):
+class _UniqueValidationHandler(ValidatorBase):
     type = 'field'
-    def add_validation_to_extension(self, field_names, fe_args, **kwargs):
-        if not field_names:
+    def create_fe_validators(self):
+        if not self.field_names:
             raise ValueError('validates_unique() must be passed at least one field name')
-        for field_to_validate in field_names:
+        for field_to_validate in self.field_names:
             valinst = _UniqueValidator(
                 cls = self.entitycls,
                 fieldname = field_to_validate
             )
-            self.validator_ext.add_validation(valinst, field_to_validate)
+            self.create_fev_meta(valinst, field_to_validate)
 
-validates_unique = ClassMutator(_UniqueValidationHandler)
+validates_unique = EntityLinker(_UniqueValidationHandler)
