@@ -1,12 +1,17 @@
+from blazeutils.testing import raises
 from nose.tools import eq_
+import sqlalchemy.sql as sasql
 from sqlalchemybwc import db
-from sqlalchemybwc.lib.decorators import one_to_none_ncm
+from sqlalchemybwc.lib.decorators import one_to_none_ncm, \
+    assert_raises_null_or_fk_exc, assert_raises_null_exc, assert_raises_fk_exc
 from sqlalchemybwc.lib.helpers import is_unique_exc, _is_unique_msg, \
     _is_unique_error_saval, _is_null_msg, _is_fk_msg, _is_check_const
 from sqlalchemybwc.lib.sql import run_app_sql, run_component_sql
+from sqlalchemybwc.lib.testing import query_to_str
 
 from sqlalchemybwc_ta.model.orm import UniqueRecord, OneToNone, Car, \
     UniqueRecordTwo, Truck, CustomerType, NoDefaults, declarative_base
+from sqlalchemybwc_ta.model.entities import Blog, Comment
 
 def test_ignore_unique():
     assert UniqueRecord.add(u'test_ignore_unique')
@@ -293,7 +298,35 @@ def test_count_and_delete_all():
     assert Car.count() == 3
     assert Car.count_by(model=u'count') == 2
     assert Car.count_where(Car.model == u'count') == 2
-    assert Car.delete_all() == 3
+    eq_(Car.delete_all(), 3)
+
+def test_query_attribute():
+    Car.delete_all()
+    c = Car.add(**{
+        'make': u'test',
+        'model': u'count',
+        'year': 2010
+    })
+    assert Car.query().count() == 1
+
+    # default usage should return all attributes
+    c = Car.query().first()
+    assert isinstance(c, Car)
+    assert c.id
+    assert c.make
+    assert c.model
+
+    # sending attribute names should give us a recordset with just those
+    # attributes
+    res = Car.query('id', 'make').first()
+    assert not isinstance(res, Car)
+    assert res.id
+    assert res.make
+    try:
+        assert res.model
+        assert False, 'expected exception'
+    except AttributeError:
+        pass
 
 def test_delete_where():
     Car.delete_all()
@@ -558,3 +591,98 @@ class TestSQLRunFuncs(object):
         run_component_sql('foo', 'run_dir_test')
         eq_(Truck.count(), 2)
         trucks = Truck.list()
+
+class TestAssertRaisesDecorators(object):
+
+    def setUp(self):
+        Comment.delete_all()
+        Blog.delete_all()
+
+    @assert_raises_null_or_fk_exc('name', 'n/a')
+    def test_ARNFKE_with_null_exc(self):
+        UniqueRecord.add(name=None)
+
+    @assert_raises_null_exc('name')
+    def test_ARNE_with_null_exc2(self):
+        UniqueRecord.add(name=None)
+
+    @raises(ValueError, 'passthru')
+    def test_ARNE_exc_passthrough(self):
+        @assert_raises_null_exc('n/a')
+        def inner():
+            raise ValueError('passthru')
+        inner()
+
+    @raises(ValueError, 'passthru')
+    def test_ARNFKE_exc_passthrough(self):
+        @assert_raises_null_or_fk_exc('n/a', 'n/a')
+        def inner():
+            raise ValueError('passthru')
+        inner()
+
+    @raises(ValueError, 'passthru')
+    def test_ARFKE_exc_passthrough(self):
+        @assert_raises_fk_exc('n/a', 'n/a')
+        def inner():
+            raise ValueError('passthru')
+        inner()
+
+    @raises(AssertionError, 'expected null exception to be raised')
+    def test_ARNE_exc_expected(self):
+        @assert_raises_null_exc('n/a')
+        def inner():
+            pass
+        inner()
+
+    @raises(AssertionError, 'expected null or FK exception to be raised')
+    def test_ARNFKE_exc_expected(self):
+        @assert_raises_null_or_fk_exc('n/a', 'n/a')
+        def inner():
+            pass
+        inner()
+
+    @raises(AssertionError, 'expected FK exception to be raised')
+    def test_ARFKE_exc_expected(self):
+        @assert_raises_fk_exc('n/a', 'n/a')
+        def inner():
+            pass
+        inner()
+
+    @assert_raises_null_or_fk_exc('blog_ident', 'ident')
+    def test_ARNFKE_with_either_exc(self):
+        # this test will generate the FK exception for SQLite
+        # but a NULL exception for other DBs
+        Comment.add(blog_ident=None)
+
+    @assert_raises_null_or_fk_exc('blog_ident', 'ident')
+    def test_ARNFKE_with_fk_exc(self):
+        Comment.add(blog_ident='abcdefg')
+
+    @assert_raises_fk_exc('blog_ident', 'ident')
+    def test_ARFKE(self):
+        Comment.add(blog_ident='abcdefg')
+
+class TestTestingHelpers(object):
+
+    def test_query_to_str_with_query(self):
+        q = db.sess.query(Blog).filter(Blog.title == u'foo').limit(10).offset(5)
+        query_str = query_to_str(q)
+        assert 'SELECT' in query_str
+        assert 'FROM blogs' in query_str
+        assert 'WHERE blogs.title = \'foo\''
+        assert 'LIMIT 10' in query_str, query_str
+        assert 'OFFSET 5' in query_str
+
+    def test_query_to_str_with_stmt(self):
+        stmt = Blog.__table__.select().where(Blog.title == u'foo').limit(10).offset(5)
+        stmt_str = query_to_str(stmt, db.engine)
+        assert 'SELECT' in stmt_str
+        assert 'FROM blogs' in stmt_str
+        assert 'WHERE blogs.title = \'foo\''
+        assert 'LIMIT 10' in stmt_str, stmt_str
+        assert 'OFFSET 5' in stmt_str
+
+    @raises('bind param (engine or connection object) required when using with an unbound statement')
+    def test_query_to_str_without_bind(self):
+        stmt = Blog.__table__.select()
+        stmt_str = query_to_str(stmt)
